/**
 * apeman app for proxy.
 * @function apemanAppProxy
 * @param {string} pattern - URL pattern to apply proxy.
 * @param {object} config - Proxy configuration.
 * @param {object} [options] - Optional settings.
 * @returns {function} - Defined app function.
 * @see https://github.com/nodejitsu/node-http-proxy
 */

'use strict'

const httpProxy = require('http-proxy')
const argx = require('argx')
const co = require('co')
const debug = require('debug')('apeman:app:proxy')

/** @lends create */
function create (pattern, config = {}, options = {}) {
  let args = argx(arguments)
  pattern = args.shift([ 'string', RegExp ])
  config = args.shift('object') || {}

  if (pattern) {
    debug(`Register proxy to patterns: ${pattern}`)
  }

  let _skip = (url) => pattern && !url.match(pattern)

  let app = co.wrap(function * proxyApp (ctx, next) {
    if (_skip(ctx.url)) {
      yield next()
      return
    }
    let { server } = ctx
    if (!ctx.proxy) {
      let proxy = create.proxy(config)
      server.on('upgrade', (req, socket, head) => {
        if (_skip(req.url)) {
          return
        }
        proxy.ws(req, socket, head)
      })
      ctx.proxy = proxy
    }

    let { req, res } = ctx
    debug(`Proxy request: ${req.url} to ${_targetString(config.target)}`)

    yield new Promise((resolve, reject) =>
      ctx.proxy.web(req, res, (err) => err ? reject(err) : resolve())
    )
  })

  Object.assign(app, {
    // Description of this app.
    $desc: 'Create proxy'
  })
  return app
}

create.proxy = function dynamic (config) {
  return httpProxy.createProxyServer(config)
}

function _targetString (target) {
  if (!target) {
    return target
  }
  if (typeof target === 'object') {
    return [ target.host, target.port ].join(':')
  }
  return target
}

module.exports = create
